# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtWebKit import *

from ..pyqgw import *


class SearchToolbar(GwFrame):
	searchSig = pyqtSignal(str, int)
	
	def __init__(self, parent=None):
		GwFrame.__init__(self, parent)
		GwHBoxLayout(self)
		
		#self.setContentsMargins(1,1,1,1)
		self.setFixedHeight(25)
		
		self.lineEdit = GwLineEdit('nep')
		self.lineEdit.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
		self.lineEdit.textEditItem.enterPressedSig.connect(self.searchNext)
		#self.lineEdit.setFixedWidth()
		self.findLabel = GwFramedTextLabel('Find:')
		self.findPrevButton = GwPushButton('< Prev')
		self.findPrevButton.clicked.connect(self.searchPrev)
		self.findNextButton = GwPushButton('Next >')
		self.findNextButton.clicked.connect(self.searchNext)
		self.closeButton = GwPushButton('close')
		self.closeButton.clicked.connect(self.closeAndClean)
		
		self.wrapChbox = GwCheckbox('Wrap')
		self.wrapChbox.setState(True)
		self.caseChbox = GwCheckbox('Case sens')
		
		self.layout().addItem(self.findLabel)
		self.layout().addItem(self.lineEdit)
		self.layout().addItem(GwSpacer(10,0))
		self.layout().addItem(self.findPrevButton)
		self.layout().addItem(GwSpacer(5,0))
		self.layout().addItem(self.findNextButton)
		self.layout().addItem(GwSpacer(10,0))
		self.layout().addItem(self.wrapChbox)
		self.layout().addItem(self.caseChbox)
		self.layout().addItem(GwSpacer(30,0))
		#self.layout().addStretch()
		self.layout().addItem(self.closeButton)
		#GwCheckbox
		#tl.setFrameStyle('none')
		
		
	def searchNext(self):
		wrap, case = 0, 0
		if self.wrapChbox.isChecked:
			wrap = QWebPage.FindWrapsAroundDocument
		if self.caseChbox.isChecked:
			case = QWebPage.FindCaseSensitively
		self.searchSig.emit(self.lineEdit.text(), wrap | case)
		
		
	def searchPrev(self):
		wrap, case = 0, 0
		if self.wrapChbox.isChecked:
			wrap = QWebPage.FindWrapsAroundDocument
		if self.caseChbox.isChecked:
			case = QWebPage.FindCaseSensitively
		self.searchSig.emit(self.lineEdit.text(), wrap | case | QWebPage.FindBackward)
		
		
	def closeAndClean(self):
		self.hide()
		self.parentWidget().layout().removeItem(self)
		#self.deleteLater()
		
		
class MainPaneCentralWidget(GwWidget):
	def __init__(self, parent=None):
		self.widget = None
		GwWidget.__init__(self, parent)
		
		
	def setWidget(self, widget):
		#print widget, self.widget
		if widget == self.widget: return
		
		if self.widget: 
			self.widget.removeEventFilter(self)
			self.widget.hide()
			#if self.widget.scene():
				##self.widget.hide()
				##self.widget.setParentItem(None)
				#self.widget.scene().removeItem(self.widget)
				
		widget.resize(self.size())
		widget.setParentItem(self)
		widget.show()
		widget.installEventFilter(self)
		self.widget = widget
		
		
	def eventFilter(self, obj, ev):
		if ev.type() == QEvent.LayoutRequest:
			self.widget.resize(self.size())
			#print 'lreq ',self.widget.size()
		return False
		
		
	def resizeEvent(self,ev):
		#print 'res', self.widget
		if self.widget:
			self.widget.resize(ev.newSize())
		
		
		
class MainPane(GwWidget):
	def __init__(self, parent=None):
		GwWidget.__init__(self, parent)
		GwVBoxLayout(self)
		self.centralWidget = MainPaneCentralWidget()
		self.searchToolbar = SearchToolbar()
		
		self.layout().addItem(self.centralWidget)
		self.layout().addItem(self.searchToolbar)
		
		searchAction = QAction(self)
		searchAction.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_F))
		searchAction.triggered.connect(self.showSearchToolbar)
		self.addAction(searchAction)
		
	def showSearchToolbar(self):
		self.layout().addItem(self.searchToolbar)
		self.searchToolbar.show()
		#self.
	 
