# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.


from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyKDE4.kdeui import *
from PyQt4.QtWebKit import *

#from PyKDE4.nepomuk import Nepomuk
#from PyKDE4.soprano import Soprano

from ..pyqgw import *

from trigview import TrigView

import time 


class EventTrigView(TrigView):
	def wheelEvent(self, ev):
		ev.ignore()
	
	
	def setHtml(self, text):
		TrigView.setHtml(self, text)
		self.setMinimumSize(QSizeF(self.page().currentFrame().contentsSize()))
		
		
	def clearSelection(self):
		''' Workaround for BUG https://bugs.webkit.org/show_bug.cgi?id=31779
		    it does not really clear selection, but moves it to first ':' 
		'''
		if not self.findText('Added', QWebPage.FindCaseSensitively | QWebPage.FindWrapsAroundDocument):
			self.findText('Removed', QWebPage.FindCaseSensitively | QWebPage.FindWrapsAroundDocument)
		self.findText(':')
		
		
	def selectLastChar(self):
		''' Workaround for BUG https://bugs.webkit.org/show_bug.cgi?id=31779
		    it does not really clear selection, but moves it to first ':' 
		'''
		while self.findText('}') : pass
		
		
class EventView(GwWidget):
	nodeSelectedSig = pyqtSignal(int, str)
	
	def __init__(self, eventIndex, trigHtml, parent=None):
		GwWidget.__init__(self, parent)
		GwVBoxLayout(self)
		
		self.eventIndex = eventIndex
		timeStr = time.strftime('%H:%M:%S')
		timeStr += ' ('+time.strftime('%H:%M:%S',time.gmtime())+' UTC)'
		self.eventHeader = GwPushButton(' [ '+str(eventIndex)+' ]   '+timeStr, align=Qt.AlignLeft)
		self.eventHeader.clicked.connect(self.toggleTrigView)
		self.trigView = EventTrigView(parent=self)
		self.trigView.setHtml(trigHtml)
		self.trigView.hide()
		self.trigView.nodeSelectedSig.connect(self.nodeSelected)
		
		self.layout().addItem(self.eventHeader)
		
		
	def toggleTrigView(self):
		if self.trigView.isVisible():
			self.layout().removeItem(self.trigView)
			self.trigView.hide()
		else:
			self.layout().addItem(self.trigView)
			self.trigView.show()
		
		
	def nodeSelected(self, node):
		self.nodeSelectedSig.emit(self.eventIndex, node)
		
		
		
class MonitoredEventsView(GwWidget):
	nodeSelectedSig = pyqtSignal(str)
	
	def __init__(self, parent=None):
		GwWidget.__init__(self, parent)
		GwVBoxLayout(self)
		self.layout().setSpacing(5)
		self.eventViews = []
		self.selectedEVIndex = None
		
		
	def event(self,ev):
		if ev.type() == QEvent.LayoutRequest:
			self.resize(self.size().width(), self.layout().minimumHeight())
		return GwWidget.event(self,ev)
		
		
	def addEvent(self, trigHtml):
		self.eventViews.append(EventView(len(self.eventViews), trigHtml))
		self.eventViews[-1].nodeSelectedSig.connect(self.nodeSelected)
		self.layout().addItem(self.eventViews[-1])
		
		
	def nodeSelected(self, eventIndex, node):
		self.nodeSelectedSig.emit(node)
		
		self.selectedEVIndex = eventIndex
		for evView in self.eventViews:
			if not evView == self.eventViews[eventIndex]:
				evView.nodeSelectedSig.disconnect(self.nodeSelected)
				evView.trigView.clearSelection()
				evView.nodeSelectedSig.connect(self.nodeSelected)
			evView.trigView.findText('', QWebPage.HighlightAllOccurrences)
			evView.trigView.findText(node, QWebPage.HighlightAllOccurrences)


	#def clearCurSelection(self):
		#if self.curEventView:
			#self.curEventView.trigView.page().findText('',QWebPage.HighlightAllOccurrences)
		#for eventView in self.eventViews.values():
			#eventView.trigView.page().findText('',QWebPage.HighlightAllOccurrences) 
