# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.


from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyKDE4.kdeui import *
from PyQt4.QtWebKit import *

#from PyKDE4.nepomuk import Nepomuk
#from PyKDE4.soprano import Soprano

from ..pyqgw import *
from ..pyqgw.delayedcall import delayedCall


from ..statement import StatementsDict
from trigview import statementsTrig, tag
from moneventview import MonitoredEventsView

class MonToolbar(GwFrame):
	def __init__(self):
		GwFrame.__init__(self)
		GwHBoxLayout(self)
		
		self.label = GwTextLabel('Captured events: 0')
		self.layout().addItem(self.label)
		

class Monitor(GwWidget):
	nodeSelectedSig = pyqtSignal(str)
	
	def __init__(self, model, parent=None):
		GwWidget.__init__(self, parent)
		GwVBoxLayout(self)
		
		self.searchFromStart = False
		
		self.toolbar = MonToolbar()
		self.layout().addItem(self.toolbar)
		
		self.monEventsView = MonitoredEventsView()
		self.scrollbox = GwVScrollBox(GwVScrollBox.Right)
		self.scrollbox.setContent(self.monEventsView)
		self.layout().addItem(self.scrollbox)
		
		self.addedStatements = []
		self.removedStatements = []
		
		self.monEventsView.nodeSelectedSig.connect(self.nodeSelectedSig)
		
		model.statementAdded.connect(self.statementAddedSlot)
		model.statementRemoved.connect(self.statementRemovedSlot)
		
		self.qtimer = QTimer()
		#self.qtimer.setSingleShot(True)
		self.qtimer.setInterval(2000)
		self.qtimer.timeout.connect(self.onTimer)
		self.qtimer.start()
	
	
	
	def search(self, searchStr, flags):
		effectiveFlags = QWebPage.FindFlags(flags & ~QWebPage.FindWrapsAroundDocument)
		
		if self.monEventsView.selectedEVIndex == None:
			if flags & QWebPage.FindWrapsAroundDocument and len(self.monEventsView.eventViews)>0:
				if flags & QWebPage.FindBackward:
					self.monEventsView.selectedEVIndex = len(self.monEventsView.eventViews)-1
					self.monEventsView.eventViews[len(self.monEventsView.eventViews)-1].trigView.selectLastChar()
				else:
					self.monEventsView.selectedEVIndex = 0
					self.monEventsView.eventViews[0].trigView.clearSelection()
			else:
				return
		
		curEvView = self.monEventsView.eventViews[self.monEventsView.selectedEVIndex]
		if curEvView.trigView.findText(searchStr, effectiveFlags):
			if not curEvView.trigView.isVisible():
				curEvView.toggleTrigView()
			return
					
		
		if flags & QWebPage.FindBackward:
			if self.monEventsView.selectedEVIndex == 0:
				if flags & QWebPage.FindWrapsAroundDocument:
					indices = range(0, len(self.monEventsView.eventViews))
					indices.reverse()
				else:
					#self.monEventsView.selectedEVIndex=None
					return
			else:
				indices = range(0, self.monEventsView.selectedEVIndex)
				indices.reverse()
		else:
			if self.monEventsView.selectedEVIndex == len(self.monEventsView.eventViews)-1:
				if flags & QWebPage.FindWrapsAroundDocument:
					indices = range(0, len(self.monEventsView.eventViews))
				else:
					#self.monEventsView.selectedEVIndex=None
					return
			else:
				indices = range(self.monEventsView.selectedEVIndex+1, len(self.monEventsView.eventViews))
		
		for i in indices:
			self.monEventsView.selectedEVIndex = i
			if flags & QWebPage.FindBackward:
				self.monEventsView.eventViews[i].trigView.selectLastChar()
			if self.monEventsView.eventViews[i].trigView.findText(searchStr, effectiveFlags):
				if not self.monEventsView.eventViews[i].trigView.isVisible():
					self.monEventsView.eventViews[i].toggleTrigView()
				#self.scrollbox.ensureVisible(self.monEventsView.eventViews[i].pos().y())
				delayedCall(400, self.scrollbox.kineticScrollTo, self.monEventsView.eventViews[i].pos().y())
				return
			
		if flags & QWebPage.FindBackward:
			if self.monEventsView.selectedEVIndex == 0:
				self.monEventsView.selectedEVIndex=None
		else:
			if self.monEventsView.selectedEVIndex == len(self.monEventsView.eventViews)-1:
				self.monEventsView.selectedEVIndex=None
		
		#print self.monEventsView.eventViews[0].trigView.page().mainFrame().scrollPosition()
		#elements = self.monEventsView.eventViews[0].trigView.page().mainFrame().findAllElements('a').toList()
		#if self.lastSearchResultElem:
			#for e in elements:
				#elements.remove(e)
				#print 'del', e, e.toPlainText()
				#if e == self.lastSearchResultElem:
					#print 'found!'
					#break
		
		#for e in elements:
			#if e.toPlainText().contains(searchStr):
				#self.lastSearchResultElem = e
				#print e, e.toPlainText(), e.geometry().center()
				#break
			
		#for self.browser.trigView.findText(searchStr)
		#self.monitor.scrollbox.ensureVisible(1000)
		
	
	def statementAddedSlot(self,s):
		self.addedStatements.append(s)
		#self.browser.addStatement()
		
	def statementRemovedSlot(self,s):
		self.removedStatements.append(s)
		
		
	def onTimer(self):
		if len(self.addedStatements)==0 and len(self.removedStatements)==0:
			return
			
		trigHtml = ''
		if len(self.addedStatements)>0:
			trigHtml += tag('green', 'Added:\n')
			trigHtml += self.parseStatements(self.addedStatements)+'\n'
		if len(self.removedStatements)>0:
			trigHtml += tag('red', 'Removed:\n')
			trigHtml += self.parseStatements(self.removedStatements)
			
		self.monEventsView.addEvent(trigHtml)
		self.toolbar.label.setText('Captured events: '+str(len(self.monEventsView.eventViews)))
		#print 'monsize',self.size()
						
		self.addedStatements = []
		self.removedStatements = []
	
	
	def parseStatements(self, statemens):
		statementsDict = StatementsDict()
		for s in statemens:
			statementsDict.addStatement(s)
			
		return statementsTrig(statementsDict)
		


