# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.


from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtWebKit import *

from PyKDE4.kdeui import *

from ..pyqgw import *
import cgi

from ..statement import qName, fullName


def tag(tag, string):
	return '<'+tag+'>'+string+'</'+tag+'>'
	
	
def qLink(string, subClass=''):
	return '<a href='+string+' class='+subClass+'>'+qName(string)+'</a>'
	
	
def statementsTrig(statementsDict, emphasizeNode=None):
	(ctxTag, subjTag, predTag, objTag) = ('o',)*4
	if emphasizeNode=='Graph': ctxTag = 'b'
	elif emphasizeNode=='Subject': subjTag = 'b'
	elif emphasizeNode=='Predicate': predTag = 'b'
	elif emphasizeNode=='Object': objTag = 'b'
		
	nstype = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type'
	trigHtml = ''
	_s = '   '
	for graph, subjects in statementsDict.iteritems():
		trigHtml += tag(ctxTag, qLink(graph, 'gray'))+' {\n'
		for subject, predicates in subjects.iteritems():
			trigHtml += _s*1+tag(subjTag, qLink(subject))+'\n'
			if nstype in predicates:
				trigHtml += _s*2+'<blue>a</blue> '+tag(objTag, qLink(predicates[nstype][0]))+'\n'
				del predicates[nstype]
			for predicate, objects in predicates.iteritems():
				trigHtml += _s*2+tag(predTag, qLink(predicate))+'\n'
				for object in objects:
					if len(object.split())>1:
						trigHtml += formatMultiLineObj(object, _s*3)
					else:
						trigHtml += _s*3+tag(objTag, qLink(object))+'\n'
		trigHtml += '}'+'\n'
	
	
	return trigHtml
		
		
def formatMultiLineObj(obj, spacer):
	paragraphs = obj.split('\n')
	result = ''
	for p in paragraphs:
		if len(paragraphs)>1:
			p += tag('light-red','\\n')
		curLine = ''
		for word in p.split():
			if len(curLine + word) > 90:
				result += spacer+curLine+'\n'
				curLine = word+' '
			else:
				curLine += word+' '
		result += spacer+curLine+'\n'
	
	return result
	
		
class TrigView(QGraphicsWebView):
	nodeSelectedSig = pyqtSignal(str)
	
	def __init__(self, text=None, parent=None):
		QGraphicsWebView.__init__(self, parent)
		
		self.cursorPos = QPoint()
		self.pageAction(QWebPage.Copy).setShortcuts(QKeySequence.Copy)
		self.addAction(self.pageAction(QWebPage.Copy))
		
		self.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
		self.page().linkClicked.connect(self.linkClicked)
		self.page().selectionChanged.connect(self.selectionChanged)
		
		if text:
			self.setHtml(text)

	
	def selectionChanged(self):
		self.page().findText('',QWebPage.HighlightAllOccurrences)
		self.nodeSelectedSig.emit('')
		
		
	def linkClicked(self, url):
		self.page().findText('',QWebPage.HighlightAllOccurrences)
		self.page().findText(qName(url.toString()), QWebPage.HighlightAllOccurrences)
		
		self.nodeSelectedSig.emit(qName(url.toString()))
		
		
	def setHtml(self, text):
		header = '''
		<html> 
		<head> 
		<style type="text/css"> 
		red {color:red} 
		green {color:green} 
		blue {color:blue} 
		light-red {color:#F4B0B0} 
		a {color:black} 
		a.gray {color:gray} 
		body {
			background-color:white;
			color:gray
		} 
		</style> 
		</head> 
		<body><pre>'''

		QGraphicsWebView.setHtml(self, header+text+'</pre></body></html>')
		
	def mouseReleaseEvent(self, ev):
		self.cursorPos = ev.pos().toPoint()
		#print self.page().mainFrame().hitTestContent().element().toPlainText()
		QGraphicsWebView.mouseReleaseEvent(self, ev)
	