# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtWebKit import *


from PyKDE4.nepomuk import Nepomuk
from PyKDE4.soprano import Soprano

from modules.pyqgw import *

from sidebar.sidebar import SideBar
from mainpane.trigview import TrigView, statementsTrig, qLink
from mainpane.monitor import Monitor
from mainpane.browser import Browser
from mainpane.mainpane import MainPane

from statement import *

#import unicodedata
import cgi

#def link(string):
	#return '<a href='+string+'>'+string+'</a>'



#def questionMsgBox(title, msgText):
	#msgBox = QMessageBox();
	#msgBox.setWindowTitle(title)
	#msgBox.setText(msgText);
	#msgBox.setIcon(QMessageBox.Question)
	#msgBox.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel);
	
	#answer = msgBox.exec_();
	#if answer == QMessageBox.Ok:
		#return True
	#else: 
		#return False
		
		
class CustomQueryDialog(QDialog):
	acceptQuerySig = pyqtSignal(str)
	
	def __init__(self, queryText='', parent=None):
		QDialog.__init__(self, parent)
		self.setWindowTitle('Please enter SPARQL query')
		
		self.textEdit = QTextEdit()
		self.textEdit.setMinimumWidth(700)
		
		buttonBox = QDialogButtonBox(QDialogButtonBox.Ok| QDialogButtonBox.Cancel)
		buttonBox.accepted.connect(self.accept)
		buttonBox.rejected.connect(self.reject)
		
		self.textEdit.setPlainText(queryText)
		
		self.setLayout(QVBoxLayout())
		self.layout().addWidget(self.textEdit)
		self.layout().addWidget(buttonBox)
	
	
	def accept(self):
		self.acceptQuerySig.emit(self.textEdit.toPlainText())
		QDialog.accept(self)
	
	

	
class NexpMain(object):
	def __init__(self):
		object.__init__(self)
		
		self.model = Nepomuk.ResourceManager.instance().mainModel()
		
		#dbusClient = Soprano.Client.DBusClient('org.kde.NepomukStorage')
		#model = dbusClient.createModel( 'main' )
		#self.nrlModel = Soprano.NRLModel( model )
		#self.nrlModel.setEnableQueryPrefixExpansion( True )
		
		self.mainWidget = GwMainWidget()
		self.mainWidget.setLayout(GwHBoxLayout())

		self.mainPane = MainPane()
		self.browser = Browser(self.mainPane)
		self.monitor = Monitor(self.model, self.mainPane)
		self.sideBar = SideBar()
		self.sideBar.setZValue(1)
		self.browser.hide()
		self.monitor.hide()
		
		self.mainWidget.layout().addItem(self.sideBar)
		self.mainWidget.layout().addItem(self.mainPane)
		
		self.curActiveTab = None
		self.previousQuery = 'select where {}'
		
		self.mainPane.searchToolbar.searchSig.connect(self.search)
		#self.browser.nodeSelectedSig.connect(self.nodeSelected)
		self.monitor.nodeSelectedSig.connect(self.nodeSelected)
		
		self.sideBar.quickButtonClickedSig.connect(self.quickButtonClicked)
		self.sideBar.monitorTab.clickedSig.connect(self.showMonitor)
		self.sideBar.tabBar.tabClickedSig.connect(self.tabClicked)
		self.sideBar.tabBar.delTabClickedSig.connect(self.delTabClicked)
		self.sideBar.tabBar.addTabSig.connect(self.addTabClicked)
		
		self.showMonitor()
		
		
	def search(self, searchStr, flags):
		if self.sideBar.monitorTab.active:
			self.monitor.search(searchStr, flags)
		else:
			self.browser.trigView.findText(searchStr, QWebPage.FindFlags(flags))
		
	
	def quickButtonClicked(self, button):
		self.addTrigView(unicode(button), self.curSelection)
	
	
	def showMonitor(self):
		if self.sideBar.monitorTab.active:
			return
			
		self.sideBar.disableQuickButtons()
		if self.curActiveTab:
			self.curActiveTab.deactivate()
		self.sideBar.monitorTab.activate()
		self.curActiveTab = self.sideBar.monitorTab
		self.mainPane.centralWidget.setWidget(self.monitor)
		
		
	def addTrigView(self,nodeType, uri):
		#print '"',uri,'"'
		#print str(uri[-1])
		#uri = uri.strip(u'\u2029\t\n')
		tabName = uri
		i=1
		while tabName in self.sideBar.tabBar.tabGroups[nodeType].tabs:
			tabName = uri+' ('+unicode(i)+')'
			i+=1
		
		uri = QUrl(fullName(uri))
		(ctx, subj, pred, obj) = (Soprano.Node(),)*4
		if nodeType=='Graph': ctx = Soprano.Node(uri)
		elif nodeType=='Subject': subj = Soprano.Node(uri)
		elif nodeType=='Predicate': pred = Soprano.Node(uri)
		elif nodeType=='Object':obj = Soprano.Node(uri)
		
		it = self.model.listStatements(subj, pred, obj, ctx)
		statementsDict = StatementsDict()
		for statement in it.allStatements():
			statementsDict.addStatement(statement)
			
		trigText = statementsTrig(statementsDict, emphasizeNode=nodeType)
		trigView = TrigView(trigText, self.browser)
		trigView.hide()
		trigView.nodeSelectedSig.connect(self.nodeSelected)
		
		self.sideBar.tabBar.tabGroups[nodeType].addTab(tabName)
		self.sideBar.tabBar.tabGroups[nodeType].tabs[tabName].trigView = trigView
		self.sideBar.tabBar.tabGroups[nodeType].tabs[tabName].tabNameEdit.selectAll()
			
			
	def addTabClicked(self, groupName):
		if groupName in ['Graph','Subject','Predicate','Object']:
			uri = ''
			while len(uri)==0:
				(uri, ok) = QInputDialog.getText(None, 'Enter Node URI','Node URI:')
				if not ok: return
				uri = unicode(uri)
			
			self.addTrigView(unicode(groupName),uri)
			
		else: # Custom Query
			customQueryDialog = CustomQueryDialog(self.previousQuery, self.mainPane.scene().views()[0])
			customQueryDialog.acceptQuerySig.connect(self.addCustomQuery)
			customQueryDialog.show()
			
	
	def addCustomQuery(self, query):
		query = unicode(query)
		self.previousQuery = query
		queriesTabGroup =  self.sideBar.tabBar.tabGroups['Custom Query']
		i=2
		tabName = 'query 1'
		while tabName in queriesTabGroup.tabs:
			tabName = 'query '+unicode(i)
			i += 1
		
		expandedQuery = ' '.join([fullName(word) for word in query.split()])
		#:
			#expandedQuery += 
		
		iterator = self.model.executeQuery(expandedQuery, Soprano.Query.QueryLanguageSparql)
		
		text = '<b>Query:</b>\n'+cgi.escape(query)+'\n\n<b>Result:</b>\n'
		i=1
		for bindingset in iterator.allBindings():
			text +='<b>Binding Set '+unicode(i)+' :</b>\n'
			i+=1
			for bindingName in bindingset.bindingNames():
				text += ' '+bindingName+' : '+qLink(bindingset[bindingName].toString())+'\n'
		
		trigView = TrigView(text, self.browser)
		trigView.hide()
		trigView.nodeSelectedSig.connect(self.nodeSelected)
		
		queriesTabGroup.addTab(tabName)
		queriesTabGroup.tabs[tabName].trigView = trigView
		queriesTabGroup.tabs[tabName].tabNameEdit.selectAll()
			
		
			
	def tabClicked(self, groupName, tabName):
		self.mainPane.centralWidget.setWidget(self.browser)
		self.sideBar.disableQuickButtons()
		if self.curActiveTab:
			self.curActiveTab.deactivate()
		self.curActiveTab = self.sideBar.tabBar.tabGroups[unicode(groupName)].tabs[unicode(tabName)]
		self.curActiveTab.activate()
		self.browser.setTrigView(self.curActiveTab.trigView)
		
		
	def delTabClicked(self, groupName, tabName):
		#print 'd'
		tab = self.sideBar.tabBar.tabGroups[unicode(groupName)].tabs[unicode(tabName)]
		if tab == self.curActiveTab:
			self.browser.setTrigView(None)
			self.sideBar.disableQuickButtons()
			self.curActiveTab = None
		self.sideBar.tabBar.tabGroups[unicode(groupName)].delTab(unicode(tabName))
		
		
	def nodeSelected(self, nodeStr):
		if nodeStr.isEmpty():
			self.sideBar.disableQuickButtons()
		else:
			self.sideBar.enableQuickButtons()
		self.curSelection = unicode(nodeStr)
