# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.


from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtSvg import QSvgRenderer

from os.path import exists

from . import GwSizedItem


#TODO: AUTOMATE CACHE CLEANING!

framePixCache = {}



bordersize = 3

partnames = [['topleft','top','topright'],['left','center','right'],['bottomleft','bottom','bottomright']]


def getFramePixmap(fileName, (w, h), prefix=''):
	if w==0 or h==0 or not exists(fileName):
		return QPixmap()
		
	if (fileName, w, h, prefix) not in framePixCache.keys():
		#print 'SvgFile(',self.filename,'):pixmapForSize exists'
		framePixCache[(fileName, w, h, prefix)] = newFramePixmap(fileName, w, h, prefix)
		
	return framePixCache[(fileName, w, h, prefix)]


def newFramePixmap(fileName, w, h, prefix):
	renderer = QSvgRenderer(fileName)
	pixmap = QPixmap(QSize(w,h))
	pixmap.fill(QColor(0,0,0,0))
	
	positions = genPositions(bordersize, w, h)
	sizes = genSizes(bordersize, w, h)
	
	painter = QPainter(pixmap)
	for i in range(3):
		for j in range(3):
			renderer.render(painter, prefix+partnames[i][j], QRectF(positions[i][j], sizes[i][j]))
			
	painter.end()
	return pixmap


def genPositions(bs, w ,h):
	return [
		[QPointF(0,0), QPointF(bs,0), QPointF(w-bs,0)],
		[QPointF(0,bs), QPointF(bs,bs), QPointF(w-bs,bs)],
		[QPointF(0,h-bs), QPointF(bs,h-bs), QPointF(w-bs,h-bs)] ]


def genSizes(bs, w ,h):
	return [
		[QSizeF(bs,bs), QSizeF(w-2*bs,bs), QSizeF(bs,bs)],
		[QSizeF(bs,h-2*bs), QSizeF(w-2*bs,h-2*bs), QSizeF(bs,h-2*bs)],
		[QSizeF(bs,bs), QSizeF(w-2*bs,bs), QSizeF(bs,bs)] ]



class GwFrameItem(QGraphicsPixmapItem, GwSizedItem):
	def __init__(self, fileName, w=0, h=0, prefix='', parent=None):
		GwSizedItem.__init__(self)
		QGraphicsPixmapItem.__init__(self, parent)
		self.fileName = fileName
		self.prefix = prefix
		if GwSizedItem.resize(self, w, h):
			self.setPixmap(getFramePixmap(fileName, (w, h), prefix))
		
		
	def setFileName(self, fileName):
		if self.fileName == fileName:
			return
		
		self.fileName = fileName
		self.setPixmap(getFramePixmap(fileName, self.size(), self.prefix))
		
		
	def resize(self, w, h):
		if GwSizedItem.resize(self, w, h):
			self.setPixmap(getFramePixmap(self.fileName, (w, h), self.prefix))
			
		