# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from textlabel import GwTextLabel
from frame import GwFrame


class LineEditItem(QGraphicsTextItem):
	focusOutSig = pyqtSignal()
	enterPressedSig = pyqtSignal()
	mousePressedSig = pyqtSignal()
	valueChangedSig = pyqtSignal(str)
	
	def __init__(self, text, parent=None):
		QGraphicsTextItem.__init__(self, text, parent)
		self.setTextInteractionFlags(Qt.TextEditorInteraction)
		self.origText = text
		self.setPlainText(text)
	
	
	def focusOutEvent(self, ev):
		QGraphicsTextItem.focusOutEvent(self, ev)
		self.focusOutSig.emit()
		
		
	def keyPressEvent(self,ev):
		if ev.key() in [Qt.Key_Return, Qt.Key_Enter]:
			self.enterPressedSig.emit()
			return
		elif ev.key() == Qt.Key_Escape:
			self.setPlainText(self.origText)
		else:
			QGraphicsTextItem.keyPressEvent(self,ev)
		self.valueChangedSig.emit(self.toPlainText())

	
	def mousePressEvent(self,ev):
		self.mousePressedSig.emit()
		QGraphicsTextItem.mousePressEvent(self,ev)



class GwLineEdit(GwFrame):
	valueChangedSig = pyqtSignal(str)
	
	def __init__(self, text='', parent=None):
		GwFrame.__init__(self, parent)
		#self.setFlag(QGraphicsItem.ItemClipsChildrenToShape, True)
		self.setFrameStyle('box')
		self.textEditItem = LineEditItem(text, self)
		#self.textEditItem.setZValue(1)
		self.textEditItem.enterPressedSig.connect(self.enterPressed)
		self.textEditItem.valueChangedSig.connect(self.valueChangedSig)
		self.setPreferredHeight(22)
		self.setMaximumHeight(self.textEditItem.boundingRect().size().height())
		self.setContentsMargins(0,0,0,0)
		
		
	def resizeEvent(self, ev):
		self.textEditItem.document().setTextWidth(self.w())
		GwFrame.resizeEvent(self,ev)
		
		
	def enterPressed(self): 
		#self.textEditItem.setTextCursor(QTextCursor(self.textEditItem.document()))
		lastCursorPos = self.textEditItem.document().rootFrame().lastCursorPosition()
		self.textEditItem.setTextCursor(lastCursorPos)
		#self.setMinimumSize(self.textEditItem.boundingRect().size())
		#self.setMaximumHeight(self.minimumHeight())
	
	
	def selectAll(self):
		cursor = self.textEditItem.textCursor()
		cursor.select(QTextCursor.BlockUnderCursor)
		self.textEditItem.setTextCursor(cursor)
	
	
	#def mousePressEvent(self, ev):
		#self.textEditItem.setFocus()
		##lastCursorPos = self.textEditItem.document().rootFrame().lastCursorPosition()
		##self.textEditItem.setTextCursor(lastCursorPos)
		##ev.ignore()
		#self.textEditItem.mousePressEvent(ev)
		
		
	def text(self):
		return self.textEditItem.toPlainText()
		
		
	#def mouseReleaseEvent(self, ev):
		#self.textEditItem.mouseReleaseEvent(ev)
		
	
	
class GwFastLineEdit(GwTextLabel):
	valueChangedSig = pyqtSignal(str)
	
	def __init__(self, text='', parent=None):
		GwTextLabel.__init__(self, text, parent)
		self.textEditItem = None
	

	def editDone(self):
		if self.textEditItem:
			self.simpleTextItem.setText(self.textEditItem.toPlainText())
			self.textEditItem.scene().removeItem(self.textEditItem)
			self.textEditItem = None
			self.valueChangedSig.emit(self.simpleTextItem.text())
		
		self.setMinimumSize(self.simpleTextItem.boundingRect().size())
		self.setMaximumHeight(self.minimumHeight())
		
		
	def startEdit(self):
		if not self.textEditItem:
			self.textEditItem = LineEditItem(self.simpleTextItem.text(), self)
			self.textEditItem.setPos(-4,-4)
			self.textEditItem.focusOutSig.connect(self.editDone)
			self.textEditItem.enterPressedSig.connect(self.editDone)
			self.textEditItem.valueChangedSig.connect(self.valueChangedSig)
		
		self.simpleTextItem.setText('')
		self.textEditItem.setFocus()
		
		#lastCursorPos = self.textEditItem.document().rootFrame().lastCursorPosition()
		#self.textEditItem.setTextCursor(lastCursorPos)
		
		
	def selectAll(self):
		if not self.textEditItem:
			self.startEdit()
		cursor = self.textEditItem.textCursor()
		cursor.select(QTextCursor.BlockUnderCursor)
		self.textEditItem.setTextCursor(cursor)
		
		
	def mousePressEvent(self,ev):
		self.startEdit()
		self.textEditItem.mousePressEvent(ev)
		#pass
		

	#def mouseReleaseEvent(self, ev):
		