# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.


from PyQt4.QtCore import *
from PyQt4.QtGui import *

from chrome import GwStdFrameItem
from generic.widget import GwWidget

from items.rectitem import GwRectItem

from layout import GwHBoxLayout
from frame import GwFrame
from textlabel import GwTextLabel, GwFramedTextLabel



class PlasmaFrame(QGraphicsPixmapItem):
	def __init__(self, name, prefix=None, parent=None):
		self.w, self.h = 0, 0
		QGraphicsPixmapItem.__init__(self, parent)
		self.plasmaFrameSvg = Plasma.FrameSvg()
		self.plasmaFrameSvg.setImagePath(name)
		if prefix:
			self.plasmaFrameSvg.setElementPrefix(prefix)
		
	def resize(self, w, h):
		#print 'FrameSvg:resizeEvent', ev.newSize()
		if w==0 or  h==0:
			return
			
		self.w, self.h = w, h
		#self.size = QSizeF(w,h)
		#self.recreatePixmap = True
		self.plasmaFrameSvg.resizeFrame(QSizeF(w,h))
		self.setPixmap(self.plasmaFrameSvg.framePixmap())
		


class GwPushButton(GwWidget):
	clicked = pyqtSignal()
	
	def __init__(self, text=None, icon=None, align=Qt.AlignCenter, parent=None):
		GwWidget.__init__(self, parent)
		self.constructed = False
		
		self.construct(text, icon, align)
	
	
	def construct(self, text, icon, align):
		#self.isDisabled = False
		self.textIconGap = 5
		self.align = align
		self.text = text
		
		#self.setFlag(QGraphicsItem.ItemClipsChildrenToShape,True)
		self.setAcceptHoverEvents(True)
		
		#svgFile = chromeFile()
		self.normalFrame = GwStdFrameItem('frame-button.svg', prefix='normal-', parent=self)
		self.focusFrame = GwStdFrameItem('frame-button.svg', prefix='focus-', parent=self)
		self.pressedFrame = GwStdFrameItem('frame-button.svg', prefix='pressed-', parent=self)
		
		#self.normalFrame = PlasmaFrame('widgets/button', 'normal', self)
		#self.focusFrame = PlasmaFrame('widgets/button', 'active', self)
		#self.pressedFrame = PlasmaFrame('widgets/button', 'pressed', self)
		#self.normalFrame.setPos(-1.0,-1.0)
		#self.focusFrame.setPos(-1.0,-1.0)
		#self.pressedFrame.setPos(-1.0,-1.0)
		self.focusFrame.hide()
		self.pressedFrame.hide()
		#self.focusFrame.setZValue(1)
		#self.pressedFrame.setZValue(1)
		
		
		minW = 0
		minH = 0
		self.icon = None
		if icon:
			icon.setParentItem(self)
			#self.icon.setZValue(2)
			#self.icon.setPos(2,2)
			#textPosOffset = self.icon.w + 10
			minW += icon.w()
			minH = icon.w()
			self.icon = icon
		
		self.textItem = None
		if text and len(text)>0:
			self.textItem = QGraphicsSimpleTextItem(text, self)
			tSize = self.textItem.boundingRect().size()
			minW += tSize.width()
			minH = max(minH, tSize.height())
		#self.textItem.setPos(textPosOffset,4)
		
		if icon and self.textItem: minW += self.textIconGap
		self.setMinimumSize(minW+6, minH+6)
		self.setPreferredSize(minW+6, minH+6)
		self.setMaximumHeight((minH+6)*2)
		self.constructed = True
			
	
	def setEnabled(self, val):
		if self.isEnabled() == val:
			return
			
		if val==True:
			self.disabledVeil.scene().removeItem(self.disabledVeil)
			self.disabledVeil = None
		else:
			self.disabledVeil = GwRectItem(self)
			#self.disabledVeil.setFlag(QGraphicsItem.ItemIgnoresParentOpacity)
			self.disabledVeil.setRect(3,3,self.w()-6,self.h()-6)
			self.disabledVeil.setColor(QColor(190,190,190))
			self.disabledVeil.setOpacity(0.9)
		
		QGraphicsWidget.setEnabled(self, val)
	
	
	def hoverEnterEvent(self, event):
		self.focusFrame.show()
		
		
	def hoverLeaveEvent(self, event):
		self.focusFrame.hide()

			
	def resizeEvent(self, ev):
		w, h = ev.newSize().width(), ev.newSize().height()
		
		if not self.isEnabled():
			self.disabledVeil.resize(w-6,h-6)
			
		self.focusFrame.resize(w,h)#+2, h+2)
		self.pressedFrame.resize(w,h)#+2, h+2)
		self.normalFrame.resize(w,h)#++2, h+2)
		
		#self.rect = QRectF(0,0,w,h)
		#print 'bu ', w ,h 
		
		if not self.icon:
			if not self.textItem:
				return
			else:
				textY = (h-self.textItem.boundingRect().height())/2
				if self.align == Qt.AlignLeft:
					textX = 4
				elif self.align == Qt.AlignCenter:
					textX = (w-self.textItem.boundingRect().width())/2
				self.textItem.setPos(textX, textY)
		else:
			(iconW, iconH) = self.icon.size()
			if not self.textItem:
				iconY = (h-iconH)/2
				if self.align == Qt.AlignLeft:
					iconX = 4
				elif self.align == Qt.AlignCenter:
					iconX = (w-iconW)/2
				self.icon.setPos(iconX, iconY)
				
			else: # self.textItem and self.icon
				iconY = (h-iconH)/2
				textY = (h -self.textItem.boundingRect().height())/2
				if self.align == Qt.AlignLeft:
					iconX = 4
					textX = iconX+iconW+self.textIconGap#(w - (self.icon.w+textW))/2 + self.icon.w + 1
				elif self.align == Qt.AlignCenter:
					textW = self.textItem.boundingRect().width()
				
					iconX = (w - (iconW+self.textIconGap+textW))/2
					textX = iconX+iconW+self.textIconGap#(w - (self.icon.w+textW))/2 + self.icon.w + 1
				
				self.icon.setPos(iconX, iconY)
				self.textItem.setPos(textX, textY)
			

	def setIconSize(self, size):
		if not self.icon:
			print 'GPushButton.setIconSize(): no icon to set size'
			return
			
		self.icon.resize(size)


	def mousePressEvent(self,ev):
		if ev.button() == Qt.LeftButton:
			self.focusFrame.hide()
			self.pressedFrame.show()
		else:
			ev.ignore()
		
		
	def mouseReleaseEvent(self, ev):
		if ev.button() == Qt.LeftButton:
			self.pressedFrame.hide()
			self.clicked.emit()
			#print ev.pos(), self.contains(ev.pos())
			#if self.contains(ev.pos()):
				#self.focusFrame.show()
			#else:
				#self.normalFrame.show()
			
			

class GwNamedButton(GwPushButton):
	namedClicked = pyqtSignal(str)
	
	def __init__(self, text=None, icon=None, align=Qt.AlignCenter, parent=None):
		GwPushButton.__init__(self, text, icon, align, parent)
		self.clicked.connect(self.emitNamedClick)
		
	def emitNamedClick(self):
		self.namedClicked.emit(self.text)
		
		
#class anbut(GPushButtonWidget):
	#def __init__(self, text=None,icon=None,parent=None):
		#GPushButtonWidget.__init__(self,text,icon,parent)
		#self.item.clicked.connect(self.anim)
		
	#def anim(self):
		#self.animation = QPropertyAnimation(self, "geometry");
		#self.animation.setDuration(4000);
		#self.animation.setKeyValueAt(0,self.geometry());
		#self.animation.setKeyValueAt(0.8, QRectF(self.pos(), self.size()+QSizeF(30,30)));
		#self.animation.setKeyValueAt(1,self.geometry());
		#self.animation.start()
 	



#class GwIcon(GwWidget):
	#def __init__(self, icon, parent=None):
		#GwWidget.__init__(self, parent)
		#icon.setParentItem(self)
		#self.icon = icon
		
	#def resizeEvent(self, ev):
		#self.icon.qResize(ev.newSize())

