# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyKDE4.kdeui import *

from ..pyqgw import *

from tabgroup import TabGroup

class MonitorTab(GwFrame):
	clickedSig = pyqtSignal()
	
	def __init__(self):
		GwFrame.__init__(self)
		#self.setFlag(QGraphicsItem.ItemIgnoresParentOpacity)
		self.setFixedHeight(20)
		
		self.setAcceptHoverEvents(True)
		
		self.icon = GwIconItem(KIcon("system-search"), 16, 16, self)
		self.label = GwTextLabel('Monitor', self)
		
		self.icon.setZValue(1)
		self.label.setZValue(1)
		
		self.icon.setPos(2,2)
		self.label.setPos(22,2)
		
		self.setMinimumWidth(self.label.minimumWidth()+22)
		
		self.active = False
		
		
	def hoverEnterEvent(self, ev):
		if not self.active:
			self.setFrameStyle('white','raised')
		
		
	def hoverLeaveEvent(self, ev):
		if not self.active:
			self.setFrameStyle('white','plain')
		
		
	def mousePressEvent(self, ev):
		self.clickedSig.emit()
		
		
	def activate(self):
		self.active = True
		self.setFrameStyle('white','active')
		
		
	def deactivate(self):
		self.active = False
		self.setFrameStyle('white','plain')
		

class TabBar(GwVScrollBox):
	tabClickedSig = pyqtSignal(str, str)
	delTabClickedSig = pyqtSignal(str, str)
	addTabSig = pyqtSignal(str)
	
	def __init__(self):
		self.tabGroups = {}
		GwVScrollBox.__init__(self)
	
		self.content = GwMinSizeWidget()
		self.content.setLayout(GwVBoxLayout())
		
		self.setAcceptHoverEvents(True)
		
		self.setContent(self.content)
	
		self._addTabGroup('Graph')
		self._addTabGroup('Subject')
		self._addTabGroup('Predicate')
		self._addTabGroup('Object')
		self._addTabGroup('Custom Query')
		
		
	def _addTabGroup(self, groupName):
		tabGroup = TabGroup(groupName)
		tabGroup.tabClickedSig.connect(self.tabClickedSig)
		tabGroup.delTabClickedSig.connect(self.delTabClickedSig)
		tabGroup.addTabSig.connect(self.addTabSig)
		tabGroup.header.setFixedWidth(self.size().width())
		self.tabGroups[groupName] = tabGroup
		self.content.layout().addItem(tabGroup)
	
	
	def resizeEvent(self,ev):
		GwVScrollBox.resizeEvent(self,ev)
		
		for tabGroup in self.tabGroups.values():
			tabGroup.header.setMaximumWidth(self.clipBox.w()-6)
			tabGroup.header.setMinimumWidth(self.clipBox.w()-6)
		
	
	def clipSizesChanged(self):
		self.clipBox.resize(self.content.size().width(),self.clipBox.size().height())
		
		
	def hoverEnterEvent(self, ev):
		self.clipBox.sizesChangedSig.connect(self.clipSizesChanged)
		self.clipSizesChanged()
		for tabGroup in self.tabGroups.values():
			for tab in tabGroup.tabs.values():
				tab.delButton.show()
		
		
	def hoverLeaveEvent(self, ev):
		self.clipBox.sizesChangedSig.disconnect(self.clipSizesChanged)
		self.reLayout()
		for tabGroup in self.tabGroups.values():
			for tab in tabGroup.tabs.values():
				tab.delButton.hide()
		


class SideBar(GwFrame):
	quickButtonClickedSig = pyqtSignal(str)
	
	def __init__(self):
		GwFrame.__init__(self)
		self.setContentsMargins(0,0,0,0)
		self.setFrameStyle('white','raised')
		GwVBoxLayout(self)
		self.setFixedWidth(200)
		
		
		qButtonsFrame = GwFrame()
		qButtonsFrame.setFrameStyle('white','raised')
		#qButtonsFrame.setFixedHeight(100)
		GwVBoxLayout(qButtonsFrame)
		self.layout().addItem(qButtonsFrame)
		qButtonsFrame.layout().addItem(GwTextLabel('Get as:'))
		
		self.quickButtons = {}
		for nodeName in ['Graph','Subject','Predicate','Object']:
			self.quickButtons[nodeName] = GwNamedButton(nodeName)
			#self.quickButtons[nodeName].setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Fixed)
			#self.quickButtons[nodeName].setMaximumWidth(500)
			self.quickButtons[nodeName].namedClicked.connect(self.quickButtonClickedSig)
			qButtonsFrame.layout().addItem(self.quickButtons[nodeName])
			
		
		self.layout().addItem(GwSpacer(10,10))
		
		self.monitorTab = MonitorTab()
		self.layout().addItem(self.monitorTab)
		
		self.disableQuickButtons()
		self.tabBar = TabBar()
		self.tabBar.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding)
		self.layout().addItem(self.tabBar)
		
		#tabGroup.tabClickedSig.connect(self.tabClickedSig)
		#self.layout().addSpacerItem(QSpacerItem(0,0,QSizePolicy.Minimum,QSizePolicy.Expanding))
		
	def disableQuickButtons(self):
		for button in self.quickButtons.values():
			button.setEnabled(False)
			
	def enableQuickButtons(self):
		for button in self.quickButtons.values():
			button.setEnabled(True)
			