# -*- coding: utf-8 -*-

# This file is part of NExp. NExp is a tool for exploring Nepomuk database.

# Copyright (C) 2010  Evstifeev Roman <someuniquename@gmail.com>

# NExp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# NExp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with NExp.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyKDE4.kdeui import *

from ..pyqgw import *

from tab import Tab

class TabGroupHeader(GwFrame):
	def __init__(self, name):
		GwFrame.__init__(self)
		
		self.setLayout(GwHBoxLayout())
		
		self.setFlag(QGraphicsItem.ItemIgnoresParentOpacity)
		
		
		self.newTabButton = GwPushButton(icon=GwIconItem(KIcon('tab-new'),16,16))
		label = GwTextLabel(name)
		self.layout().addItem(label)
		
		self.layout().addStretch()
		self.layout().addItem(self.newTabButton)
		



class TabGroup(GwFrame):
	tabClickedSig = pyqtSignal(str, str)
	delTabClickedSig = pyqtSignal(str, str)
	addTabSig = pyqtSignal(str)
	
	def __init__(self, groupName):
		GwFrame.__init__(self)
		self.setLayout(GwVBoxLayout())#QGraphicsLinearLayout(Qt.Vertical))
		
		self.groupName = groupName
		self.header = TabGroupHeader(groupName)
		self.header.newTabButton.clicked.connect(self.addTabClicked)
		
		
		self._preContent = QGraphicsWidget()
		self._preContent.setLayout(GwHBoxLayout())
		#self._preContent.layout().addSpacerItem(QSpacerItem(5,5,QSizePolicy.Fixed))
		#GwSpacer
		#self.layout().addStretch(30)
		self.content = GwWidget()
		self.content.setLayout(GwVBoxLayout())#QGraphicsLinearLayout(Qt.Vertical))
		self.content.layout().setContentsMargins(1,1,1,1)
		self.content.layout().setSpacing(2)
		
		self.layout().addItem(self.header)
		self.layout().addItem(self._preContent)
		self._preContent.layout().addItem(GwSpacer(22,0))
		self._preContent.layout().addItem(self.content)
		
		self.tabs = {}
		
		#self.setFlag(QGraphicsItem.ItemDoesntPropagateOpacityToChildren)
		self.setOpacity(0.4)
		
		
	def tabClicked(self, tabName):
		self.tabClickedSig.emit(self.groupName, tabName)
		
	
	def delTabClicked(self, tabName):
		self.delTabClickedSig.emit(self.groupName, tabName)
		
		
	def delTab(self, tabName):
		self.tabs[unicode(tabName)].deleteLater()
		del self.tabs[unicode(tabName)]
		 
		 
	def addTabClicked(self):
		self.addTabSig.emit(self.groupName)
		
			
	def addTab(self, tabName):
		self.tabs[tabName] = Tab(tabName)
		self.content.layout().addItem(self.tabs[tabName])
		self.tabs[tabName].clickedSig.connect(self.tabClicked)
		self.tabs[tabName].delClickedSig.connect(self.delTabClicked)
		
		
